/*
 * AutomatonFeatures.h
 *
 *  Created on: Jun 19, 2014
 *      Author: Jan Travnicek
 */

#ifndef AUTOMATON_FEATURES_H_
#define AUTOMATON_FEATURES_H_

#include <common/DefaultStateType.h>
#include <common/DefaultEpsilonType.h>
#include <common/DefaultSymbolType.h>
#include <common/DefaultRankType.h>

namespace automaton {

enum class FEATURES {
	MULTI_INITIAL_STATE_EPSILON_NFA,
	EPSILON_NFA,
	MULTI_INITIAL_STATE_NFA,
	NFA,
	DFA,
	COMPACT_NFA,
	EXTENDED_NFA,
	DPDA,
	SINGLE_POP_DPDA,
	INPUT_DRIVEN_DPDA,
	INPUT_DRIVEN_NPDA,
	VISIBLY_PUSHDOWN_DPDA,
	VISIBLY_PUSHDOWN_NPDA,
	REAL_TIME_HEIGHT_DETERMINISTIC_DPDA,
	REAL_TIME_HEIGHT_DETERMINISTIC_NPDA,
	NPDA,
	SINGLE_POP_NPDA,
	ONE_TAPE_DTM,
	DFTA,
	NFTA
};

class Automaton;
class AutomatonBase;
enum class Shift;

template<class SymbolType = DefaultSymbolType, class EpsilonType = DefaultEpsilonType, class StateType = DefaultStateType >
class MultiInitialStateEpsilonNFA;
template<class SymbolType = DefaultSymbolType, class EpsilonType = DefaultEpsilonType, class StateType = DefaultStateType >
class EpsilonNFA;
template<class SymbolType = DefaultSymbolType, class StateType = DefaultStateType >
class NFA;
template<class SymbolType = DefaultSymbolType, class StateType = DefaultStateType >
class MultiInitialStateNFA;
template<class SymbolType = DefaultSymbolType, class StateType = DefaultStateType >
class DFA;
template<class SymbolType = DefaultSymbolType, class StateType = DefaultStateType >
class CompactNFA;
template<class SymbolType = DefaultSymbolType, class StateType = DefaultStateType >
class ExtendedNFA;
template<class InputSymbolType = DefaultSymbolType, class EpsilonType = DefaultEpsilonType, class PushdownStoreSymbolType = DefaultSymbolType, class StateType = DefaultStateType >
class DPDA;
template<class InputSymbolType = DefaultSymbolType, class PushdownStoreSymbolType = DefaultSymbolType, class StateType = DefaultStateType >
class InputDrivenDPDA;
template<class InputSymbolType = DefaultSymbolType, class PushdownStoreSymbolType = DefaultSymbolType, class StateType = DefaultStateType >
class InputDrivenNPDA;
template<class InputSymbolType = DefaultSymbolType, class PushdownStoreSymbolType = DefaultSymbolType, class StateType = DefaultStateType >
class VisiblyPushdownDPDA;
template<class InputSymbolType = DefaultSymbolType, class PushdownStoreSymbolType = DefaultSymbolType, class StateType = DefaultStateType >
class VisiblyPushdownNPDA;
template<class InputSymbolType = DefaultSymbolType, class EpsilonType = DefaultEpsilonType, class PushdownStoreSymbolType = DefaultSymbolType, class StateType = DefaultStateType >
class RealTimeHeightDeterministicDPDA;
template<class InputSymbolType = DefaultSymbolType, class EpsilonType = DefaultEpsilonType, class PushdownStoreSymbolType = DefaultSymbolType, class StateType = DefaultStateType >
class RealTimeHeightDeterministicNPDA;
template<class InputSymbolType = DefaultSymbolType, class EpsilonType = DefaultEpsilonType, class PushdownStoreSymbolType = DefaultSymbolType, class StateType = DefaultStateType >
class NPDA;
template<class InputSymbolType = DefaultSymbolType, class OutputSymbolType = DefaultSymbolType, class EpsilonType = DefaultEpsilonType, class PushdownStoreSymbolType = DefaultSymbolType, class StateType = DefaultStateType >
class NPDTA;
template<class InputSymbolType = DefaultSymbolType, class EpsilonType = DefaultEpsilonType, class PushdownStoreSymbolType = DefaultSymbolType, class StateType = DefaultStateType >
class SinglePopDPDA;
template<class InputSymbolType = DefaultSymbolType, class EpsilonType = DefaultEpsilonType, class PushdownStoreSymbolType = DefaultSymbolType, class StateType = DefaultStateType >
class SinglePopNPDA;
template<class SymbolType = DefaultSymbolType, class StateType = DefaultStateType >
class OneTapeDTM;
template<class SymbolType = DefaultSymbolType, class RankType = DefaultRankType, class StateType = DefaultStateType >
class DFTA;
template<class SymbolType = DefaultSymbolType, class RankType = DefaultRankType, class StateType = DefaultStateType >
class NFTA;

} /* namespace automaton */

#endif /* AUTOMATON_FEATURES_H_ */
