#ifndef _XML_REGISTRATION_HPP_
#define _XML_REGISTRATION_HPP_

#include <core/xmlApi.hpp>

#include <registry/XmlComposerRegistry.hpp>
#include <registry/XmlParserRegistry.hpp>

#include <registry/XmlRegistry.h>
#include <registry/AlgorithmRegistry.hpp>

namespace xml {

class Parse {
public:
	static std::shared_ptr < abstraction::OperationAbstraction > abstractionFromTokens ( ext::deque < sax::Token > && tokens ) {
		return abstraction::XmlRegistry::getXmlParserAbstraction ( tokens [ 0 ].getData ( ) );
	}

};

class Compose {
public:
	template < class Type >
	static std::shared_ptr < abstraction::OperationAbstraction > abstractionFromType ( const Type & ) {
		return abstraction::XmlRegistry::getXmlComposerAbstraction ( ext::to_string < Type > ( ) );
	}

};

} /* namespace xml */

namespace registration {

template < class Type >
class XmlReaderRegister {
public:
	XmlReaderRegister ( ) {
		abstraction::XmlParserRegistry::registerXmlParser < Type > ( );
	}
};

template < class Type >
class XmlWriterRegister {
public:
	XmlWriterRegister ( ) {
		abstraction::XmlComposerRegistry::registerXmlComposer < Type > ( );
		abstraction::AlgorithmRegistry::registerWrapper < xml::Compose, const Type & > ( xml::Compose::abstractionFromType, std::array < std::string, 1 > { { "arg0" } } );
	}
};

template < class Group, class Type >
class XmlRegisterTypeInGroup {
public:
	XmlRegisterTypeInGroup ( ) {
		core::xmlApi < Group >::template registerXmlReader < Type > ( );
		core::xmlApi < Group >::template registerXmlWriter < Type > ( );
	}
};

} /* namespace registration */

#endif // _XML_REGISTRATION_HPP_
